use dioxus::prelude::*;
use dioxus_material_icons::MaterialIcon;
use qrc::QRCode;

use crate::{
    api::get_item,
    fetch_api,
    page::{item_detail::EventButton, to_human_date},
    qrscan::image_buffer_to_data_url,
    Route,
};

#[component]
pub fn TransactionPage(id: String) -> Element {
    let id = use_signal(|| id.clone());
    let transaction = use_resource(move || async move {
        crate::api::API::get_transaction(id.read().to_string()).await
    });

    let qr = QRCode::from_string(id.read().clone()).to_png(512);
    let qr_code_src = image_buffer_to_data_url(&qr).unwrap();

    rsx! {
        match transaction.value()() {
            Some(transaction) => rsx! {
                div {
                    class: "max-w-4xl mx-auto p-4 space-y-6",

                    // HEADER CARD
                    div {
                        class: "flex flex-row items-center space-x-6 p-4 rounded-lg bg-white dark:bg-gray-800 shadow-md",

                    img {
        class: "w-24 h-24 rounded-md border border-gray-300 dark:border-gray-700 bg-white dark:bg-gray-900 shadow-md",
        src: qr_code_src,
        alt: "Transaction QR code"
    }

                        div {
                            class: "flex flex-col space-y-1",

                            p { class: "text-xl font-semibold text-gray-900 dark:text-gray-100", "{get_item(&transaction.item).unwrap().name}" }
                            p { class: "text-md text-gray-600 dark:text-gray-300", "{get_item(&transaction.item).unwrap().get_variant(&transaction.variant).unwrap().name}" }
                            p { class: "text-sm text-gray-500 dark:text-gray-400", {to_human_date(transaction.timestamp)} }
                            if transaction.expired && transaction.consumed.is_none() {
                                span {
                                    class: "inline-block bg-red-100 text-red-700 text-xs font-semibold px-2 py-1 rounded-full uppercase tracking-wide",
                                    "Expired"
                                }
                            }
                        }
                    },

                    // CONSUME INFO CARD or Consume Button
                    if let Some(consumed) = transaction.consumed.as_ref() {
                        div {
                            class: "p-4 rounded-lg bg-blue-50 dark:bg-blue-900 shadow-md flex flex-col space-y-3",

                            h3 { "Consumed" }

                            IconLabel {
                                icon: "location_on".to_string(),
                                label: "Destination".to_string(),
                                value: consumed.destination.clone()
                            }
                            IconLabel {
                                icon: "attach_money".to_string(),
                                label: "Price".to_string(),
                                value: format!("{:.2}", consumed.price)
                            }
                            IconLabel {
                                icon: "schedule".to_string(),
                                label: "Consumed At".to_string(),
                                value: to_human_date(consumed.timestamp)
                            }
                        }
                    } else {
                        EventButton {
                            icon: "check_circle".to_string(),
                            title: "Consume".to_string(),
                            onclick: move |_| {
                                navigator().push(Route::ConsumePage {
                                    id: transaction.uuid.clone(),
                                    item: transaction.item.clone(),
                                    variant: transaction.variant.clone(),
                                });
                            }
                        }
                    },

                    // OTHER INFO LIST
                    div {
                        class: "space-y-3",

                        IconLabel {
                            icon: "attach_money".to_string(),
                            label: "Price".to_string(),
                            value: format!("{:.2}", transaction.price)
                        }

                        if let Some(origin) = transaction.origin.as_ref() {
                            IconLabel {
                                icon: "home".to_string(),
                                label: "Origin".to_string(),
                                value: origin.clone()
                            }
                        }

                        if let Some(location) = transaction.location.as_ref() {
                            IconLabel {
                                icon: "place".to_string(),
                                label: "Location".to_string(),
                                value: location.name.clone()
                            }
                        }

                        if let Some(note) = transaction.note.as_ref() {
                            if !note.is_empty() {
                                IconLabel {
                                    icon: "note".to_string(),
                                    label: "Note".to_string(),
                                    value: note.clone()
                                }
                            }
                        }

                        if let Some(quanta) = transaction.quanta {
                            IconLabel {
                                icon: "inventory_2".to_string(),
                                label: "Quantity".to_string(),
                                value: quanta.to_string()
                            }
                        }

                        if let Some(properties) = transaction.properties.as_ref() {
                            IconLabel {
                                icon: "info".to_string(),
                                label: "Properties".to_string(),
                                value: serde_json::to_string_pretty(properties).unwrap_or_else(|_| "{}".to_string())
                            }
                        }
                    }
                }
            },
            None => rsx! {
                p {
                    class: "text-center text-gray-500 dark:text-gray-400 p-6",
                    "Loading transaction..."
                }
            }
        }
    }
}

#[component]
pub fn IconLabel(icon: String, label: String, value: String) -> Element {
    rsx! {
        div {
            class: "flex items-center space-x-4 p-2 rounded-md bg-gray-50 dark:bg-gray-800 hover:bg-gray-100 dark:hover:bg-gray-700 transition-colors duration-200 shadow-sm",

                MaterialIcon {
                    name: icon,
                    size: 24,
                },

            div {
                class: "flex flex-col",

                span {
                    class: "text-xs font-semibold uppercase tracking-wide text-gray-500 dark:text-gray-400",
                    "{label}"
                },

                span {
                    class: "text-base font-bold text-gray-900 dark:text-gray-100",
                    "{value}"
                }
            }
        }
    }
}
