use dioxus::prelude::*;
use serde::{Deserialize, Serialize};
use std::collections::HashMap;

use crate::store::save_to_local_storage;

#[derive(Default, Serialize, Deserialize, Clone, PartialEq)]
pub struct Credentials {
    pub instance_url: String,
    pub token: String,
}

impl Credentials {
    pub fn empty(&self) -> bool {
        self.instance_url.is_empty() && self.token.is_empty()
    }
}

#[derive(Default, Clone)]
struct FormErrors {
    instance_url: Option<String>,
    token: Option<String>,
}

#[component]
pub fn SetupPage() -> Element {
    let mut form_data = use_signal(Credentials::default);
    let mut form_data2 = form_data.clone();
    let mut form_errors = use_signal(FormErrors::default);
    let submitted = use_signal(|| false);

    let mut validate_form = move || {
        let mut errors = FormErrors::default();
        if form_data2.read().instance_url.is_empty() {
            errors.instance_url = Some("Instance URL is required".to_string());
        }
        if form_data2.read().token.is_empty() {
            errors.token = Some("Token is required".to_string());
        }

        form_errors.set(errors.clone());
        errors.instance_url.is_none() && errors.token.is_none()
    };

    let on_submit = move |evt: FormEvent| {
        evt.prevent_default();

        if validate_form() {
            let creds = Credentials::default();

            spawn(async move {
                let api = crate::api::API::new(creds.instance_url.clone()).await;
                *crate::API.write() = Some(api);
            });

            save_to_local_storage("creds", form_data.read().clone());
        }
    };

    rsx! {
        div {
            class: "min-h-screen flex flex-col justify-center items-center px-4",

            h1 {
                class: "text-3xl font-bold mb-8 text-neutral-100",
                "Setup Page"
            }

            form {
                class: "w-full max-w-md bg-stone-900 rounded-lg shadow-lg p-6 space-y-6",
                onsubmit: on_submit,

                div {
                    class: "flex flex-col",
                    label {
                        class: "mb-2 font-semibold text-gray-700 dark:text-gray-300",
                        "Instance URL:"
                    }
                    input {
                        class: "border border-gray-300 dark:border-gray-700 rounded-md px-3 py-2 focus:outline-none focus:ring-2 focus:ring-blue-500 dark:bg-gray-700 dark:text-gray-100",
                        value: "{form_data.read().instance_url}",
                        oninput: move |evt| {
                            let mut form_data = form_data.write();
                            form_data.instance_url = evt.value();
                        }
                    }
                    {form_errors.read().instance_url.as_ref().map(|error| rsx! {
                        p { class: "text-red-600 mt-1 text-sm", "{error}" }
                    })}
                }

                div {
                    class: "flex flex-col",
                    label {
                        class: "mb-2 font-semibold text-gray-700 dark:text-gray-300",
                        "Token:"
                    }
                    input {
                        class: "border border-gray-300 dark:border-gray-700 rounded-md px-3 py-2 focus:outline-none focus:ring-2 focus:ring-blue-500 dark:bg-gray-700 dark:text-gray-100",
                        r#type: "password",
                        value: "{form_data.read().token}",
                        oninput: move |evt| {
                            let mut form_data = form_data.write();
                            form_data.token = evt.value();
                        }
                    }
                    {form_errors.read().token.as_ref().map(|error| rsx! {
                        p { class: "text-red-600 mt-1 text-sm", "{error}" }
                    })}
                }

                button {
                    type: "submit",
                    class: "cursor-pointer w-full bg-blue-600 hover:bg-blue-700 focus:ring-4 focus:ring-blue-300 text-white font-semibold py-2 rounded-md transition",
                    "Complete Setup"
                }
            }

            if *submitted.read() {
                div {
                    class: "mt-6 bg-green-100 dark:bg-green-800 text-green-800 dark:text-green-200 rounded-md p-4 max-w-md w-full text-center font-semibold",
                    h3 { "Setup Complete!" }
                }
            }
        }
    }
}
